@@ -26,6 +26,7 @@ with _add_dll_dir():
     from .classad2_impl import _version as version
 
     from ._class_ad import ClassAd
+    # This should not be part of the API...
     from ._class_ad import _convert_local_datetime_to_utc_ts
     from ._value import Value
     from ._expr_tree import ExprTree
@@ -36,6 +37,7 @@ with _add_dll_dir():
     from ._class_ad import _parseNext as parseNext
     from ._class_ad import _quote as quote
     from ._class_ad import _unquote as unquote
+    from ._class_ad import _lastError as lastError
 
     # For compability with version 1.
     from ._parser_type import ParserType as Parser
