@@ -2,7 +2,10 @@
 
 # Build RPMs and debs for Linux in the current directory using the
 # source in the named directory
-if [ -f /etc/debian_version -o -f /etc/redhat-release -o -f /etc/system-release ]; then
+. /etc/os-release
+if [ $ID = 'almalinux' ] || [ $ID = 'amzn' ] || [ $ID = 'centos' ] ||
+   [ $ID = 'debian' ] || [ $ID = 'fedora' ] || [ $ID = 'opensuse-leap' ] ||
+   [ $ID = 'ubuntu' ]; then
 
     # Locate the source directory
     if [ $# -eq 1 -a -d $1 ]; then
@@ -31,8 +34,7 @@ if [ -f /etc/debian_version -o -f /etc/redhat-release -o -f /etc/system-release
     # Call the official build scripts
     if [ -f /etc/debian_version ]; then
         exec ${src_dir}/nmi_tools/glue/build/build_uw_deb.sh
-    fi
-    if [ -f /etc/redhat-release -o -f /etc/system-release ]; then
+    else
         exec ${src_dir}/nmi_tools/glue/build/build_uw_rpm.sh
     fi
 fi
