@@ -23,6 +23,7 @@
 #include "classad/lexer.h"
 #include "classad/util.h"
 #include "classad/classad.h"
+#include <charconv>
 
 using std::string;
 using std::vector;
@@ -431,22 +432,16 @@ tokenizeNumber (void)
 
 	if( numberType == INTEGER ) {
 		cut( );
-		long long l;
-		int base = 0;
+		long long l = 0;
 		if ( _useOldClassAdSemantics || jsonLex ) {
 			// Old ClassAds and JSON don't support octal or hexidecimal
 			// representations for integers.
-			base = 10;
 			if ( lexBuffer[0] == '0' && lexBuffer.length() > 1 ) {
 				tokenType = LEX_TOKEN_ERROR;
 				return( tokenType );
 			}
 		}
-#ifdef WIN32
-		l = _strtoi64( lexBuffer.c_str(), NULL, base );
-#else
-		l = strtoll( lexBuffer.c_str(), NULL, base );
-#endif
+		std::ignore = std::from_chars(lexBuffer.data(), lexBuffer.data() + lexBuffer.size(), l, 10);
 		integer = l;
 	} else if( numberType == REAL ) {
 		cut( );
