@@ -97,7 +97,7 @@ int CollectorDaemon::UpdateTimerId;
 
 OfflineCollectorPlugin CollectorDaemon::offline_plugin_;
 
-StringList *viewCollectorTypes;
+std::vector<std::string> viewCollectorTypes;
 
 CCBServer *CollectorDaemon::m_ccb_server;
 bool CollectorDaemon::filterAbsentAds;
@@ -186,16 +186,7 @@ CollectorDaemon::schedd_token_request(int, Stream *stream)
 
 	std::string authz_list_str;
 	ad.EvaluateAttrString(ATTR_SEC_LIMIT_AUTHORIZATION, authz_list_str);
-	std::vector<std::string> authz_bounding_set;
-	if (!authz_list_str.empty())
-	{
-		StringList authz_list(authz_list_str.c_str());
-		authz_list.rewind();
-		const char *authz_name;
-		while ( (authz_name = authz_list.next()) ) {
-			authz_bounding_set.emplace_back(authz_name);
-		}
-	}
+	std::vector<std::string> authz_bounding_set = split(authz_list_str);
 	int requested_lifetime = -1;
 	if (!ad.EvaluateAttrInt(ATTR_SEC_TOKEN_LIFETIME, requested_lifetime)) {
 		requested_lifetime = -1;
@@ -302,7 +293,6 @@ void CollectorDaemon::Init()
 	// read in various parameters from condor_config
 	CollectorName=NULL;
 	ad=NULL;
-	viewCollectorTypes = NULL;
 	UpdateTimerId=-1;
 	collectorsToUpdate = NULL;
 	Config();
@@ -560,7 +550,7 @@ CollectorDaemon::pending_query_entry_t *  CollectorDaemon::make_query_entry(
 		// for QUERY_STARTD_ADS, we look at the target type to figure which startd ad table to use
 		// if no target type, assume the slot ad table
 		if (query->LookupString(ATTR_TARGET_TYPE, target)) {
-			whichAds = get_real_startd_ad_type(target.c_str());
+			whichAds = get_realish_startd_adtype(target.c_str());
 			//?? label = AdTypeToString(whichAds);
 		}
 	} else if (whichAds == STARTD_PVT_AD && ! allow_pvt) {
@@ -1472,11 +1462,6 @@ int CollectorDaemon::receive_invalidation(int command,
     if (whichAds != (AdTypes) -1)
 		op.process_invalidation (whichAds, cad, sock);
 
-	// if the invalidation was for the STARTD ads, also invalidate startd
-	// private ads with the same query ad
-	if (command == INVALIDATE_STARTD_ADS)
-		op.process_invalidation (STARTD_PVT_AD, cad, sock);
-
     /* let the off-line plug-in invalidate the given ad */
     offline_plugin_.invalidate ( command, cad );
 
@@ -1484,7 +1469,7 @@ int CollectorDaemon::receive_invalidation(int command,
 	CollectorPluginManager::Invalidate(command, cad);
 #endif
 
-	if (viewCollectorTypes || command == INVALIDATE_STARTD_ADS || command == INVALIDATE_SUBMITTOR_ADS) {
+	if (!viewCollectorTypes.empty() || command == INVALIDATE_STARTD_ADS || command == INVALIDATE_SUBMITTOR_ADS) {
 		forward_classad_to_view_collector(command,
 										  ATTR_TARGET_TYPE,
 										  &cad);
@@ -1576,7 +1561,7 @@ int CollectorDaemon::receive_update(int command, Stream* sock)
 	CollectorEngine_ru_plugins_runtime += rt.tick(rt_last);
 #endif
 
-	if (viewCollectorTypes || command == UPDATE_STARTD_AD || command == UPDATE_SUBMITTOR_AD) {
+	if (!viewCollectorTypes.empty() || command == UPDATE_STARTD_AD || command == UPDATE_SUBMITTOR_AD) {
 		forward_classad_to_view_collector(command,
 										  ATTR_MY_TYPE,
 										  record->m_pvtAd);
@@ -1712,7 +1697,7 @@ int CollectorDaemon::receive_update_expect_ack(int command,
 		CollectorPluginManager::Update ( command, *record->m_publicAd );
 #endif
 
-		if (viewCollectorTypes || UPDATE_STARTD_AD_WITH_ACK == command) {
+		if (!viewCollectorTypes.empty() || UPDATE_STARTD_AD_WITH_ACK == command) {
 			forward_classad_to_view_collector(command,
 										  ATTR_MY_TYPE,
 										  record->m_pvtAd);
@@ -1906,8 +1891,13 @@ void CollectorDaemon::collect_op::process_invalidation (AdTypes whichAds, ClassA
 
 	CollectorEngine::HashFunc makeKey = nullptr;
 	CollectorHashTable * hTable = nullptr;
+	CollectorHashTable * hTablePvt = nullptr;
+	int num_pvt = 0;
 	bool expireInvalidatedAds = param_boolean( "EXPIRE_INVALIDATED_ADS", false );
 
+	//std::string adbuf;
+	//dprintf(D_ZKM, "process_invalidate(%d) : %s\n", whichAds, formatAd(adbuf, query, "\t", nullptr, false));
+
 	// For commands that specify the ad table type indirectly via the command int
 	// it is not necessary to specify the target type of the ad(s) to be invalidated.
 	// But when the command doesn't indicate a specific table, use the targettype
@@ -1929,6 +1919,18 @@ void CollectorDaemon::collect_op::process_invalidation (AdTypes whichAds, ClassA
 			dprintf(D_ALWAYS, "Invalidate %s failed - no target MyType specified\n", AdTypeToString(whichAds));
 			return;
 		}
+	} else if (whichAds == STARTD_AD) {
+		// INVALIDATE_STARTD_AD is used to invalidate the slot and the daemon ads
+		if (query.LookupString(ATTR_TARGET_TYPE, target_type) && YourStringNoCase(STARTD_DAEMON_ADTYPE) == target_type) {
+			whichAds = STARTDAEMON_AD;
+		}
+		collector.LookupByAdType(whichAds, hTable, makeKey);
+		if ( ! hTable) { target_type = AdTypeToString(whichAds); } // for the error message
+		else if (whichAds == STARTD_AD) { 
+			CollectorEngine::HashFunc dummy;
+			collector.LookupByAdType(STARTD_PVT_AD, hTablePvt, dummy);
+		}
+
 	} else {
 		collector.LookupByAdType(whichAds, hTable, makeKey);
 		if ( ! hTable) { target_type = AdTypeToString(whichAds); } // for the error message
@@ -1943,8 +1945,16 @@ void CollectorDaemon::collect_op::process_invalidation (AdTypes whichAds, ClassA
 
 		if( expireInvalidatedAds ) {
 			__numAds__ = collector.expire(hTable, hKey);
+			if (hTablePvt) {
+				num_pvt = collector.expire(hTablePvt, hKey);
+				if (num_pvt > __numAds__) { dprintf(D_ALWAYS, "WARNING: expired %d private ads for %d ads\n", num_pvt, __numAds__); }
+			}
 		} else {
 			__numAds__ = collector.remove(hTable, hKey, whichAds);
+			if (hTablePvt) {
+				num_pvt = collector.remove(hTablePvt, hKey, STARTD_PVT_AD);
+				if (num_pvt > __numAds__) { dprintf(D_ALWAYS, "WARNING: removed %d private ads for %d ads\n", num_pvt, __numAds__); }
+			}
 		}
 
 	} else {
@@ -1961,24 +1971,34 @@ void CollectorDaemon::collect_op::process_invalidation (AdTypes whichAds, ClassA
 
 		if (expireInvalidatedAds) {
 			collector.walkHashTable (*hTable, [this](CollectorRecord*cr){return this->expiration_scanFunc(cr);});
+			if (hTablePvt) { collector.walkHashTable (*hTablePvt, [this](CollectorRecord*cr){return this->expiration_scanFunc(cr);}); }
 			collector.invokeHousekeeper (whichAds);
 		} else if (param_boolean("HOUSEKEEPING_ON_INVALIDATE", true))  {
 			// first set all the "LastHeardFrom" attributes to low values ...
 			collector.walkHashTable (*hTable, [this](CollectorRecord*cr){return this->invalidation_scanFunc(cr);});
+			if (hTablePvt) { collector.walkHashTable (*hTablePvt, [this](CollectorRecord*cr){return this->invalidation_scanFunc(cr);}); }
 			// ... then invoke the housekeeper
 			collector.invokeHousekeeper (whichAds);
 		} else {
 			__numAds__ = collector.invalidateAds(hTable, __mytype__, query);
+			if (hTablePvt) {
+				num_pvt = collector.invalidateAds(hTablePvt, __mytype__, query);
+				if (num_pvt > __numAds__) { dprintf(D_ALWAYS, "WARNING: invalidated %d private ads for %d ads\n", num_pvt, __numAds__); }
+			}
 		}
 	}
 
-	dprintf (D_ALWAYS, "(Invalidated %d ads)\n", __numAds__ );
+	if (hTablePvt) {
+		dprintf (D_ALWAYS, "(Invalidated %d ads) + %d private ads\n", __numAds__, num_pvt );
+	} else {
+		dprintf (D_ALWAYS, "(Invalidated %d ads)\n", __numAds__ );
+	}
 
 		// Suppose lots of ads are getting invalidated and we have no clue
 		// why.  That is what the following block of code tries to solve.
 	if( __numAds__ > 1 ) {
-		dprintf(D_ALWAYS, "The invalidation query was this:\n");
-		dPrintAd(D_ALWAYS, query);
+		std::string buf;
+		dprintf(D_ALWAYS, "The invalidation query was this: %s\n", formatAd(buf, query, "\t", nullptr, false));
 	}
 }	
 
@@ -2166,20 +2186,17 @@ void CollectorDaemon::Config()
 
     tmp = param("CONDOR_VIEW_HOST");
     if (tmp) {
-        StringList cvh(tmp);
-        free(tmp);
-        cvh.rewind();
-        while (char* vhost = cvh.next()) {
-            DCCollector* vhd = new DCCollector(vhost, DCCollector::CONFIG_VIEW);
+        for (auto& vhost: StringTokenIterator(tmp)) {
+            DCCollector* vhd = new DCCollector(vhost.c_str(), DCCollector::CONFIG_VIEW);
             Sinful view_addr( vhd->addr() );
             Sinful my_addr( daemonCore->publicNetworkIpAddr() );
 
             if (my_addr.addressPointsToMe(view_addr)) {
-                dprintf(D_ALWAYS, "Not forwarding to View Server %s - self referential\n", vhost);
+                dprintf(D_ALWAYS, "Not forwarding to View Server %s - self referential\n", vhost.c_str());
                 delete vhd;
                 continue;
             }
-            dprintf(D_ALWAYS, "Will forward ads on to View Server %s\n", vhost);
+            dprintf(D_ALWAYS, "Will forward ads on to View Server %s\n", vhost.c_str());
 
             Sock* vhsock = NULL;
             if (vhd->useTCPForUpdates()) {
@@ -2193,6 +2210,7 @@ void CollectorDaemon::Config()
             vc_list.back().collector = vhd;
             vc_list.back().sock = vhsock;
         }
+        free(tmp);
     }
 
     if (!vc_list.empty()) {
@@ -2201,19 +2219,18 @@ void CollectorDaemon::Config()
         view_sock_timeslice.setMaxInterval(1200);
     }
 
-	if (viewCollectorTypes) delete viewCollectorTypes;
-	viewCollectorTypes = NULL;
+	viewCollectorTypes.clear();
 	if (!vc_list.empty()) {
-		auto_free_ptr tmp(param("CONDOR_VIEW_CLASSAD_TYPES"));
-		if (tmp) {
-			viewCollectorTypes = new StringList(tmp);
-			if (viewCollectorTypes->contains_anycase(STARTD_OLD_ADTYPE) && 
-				! viewCollectorTypes->contains_anycase(STARTD_SLOT_ADTYPE)) {
-				viewCollectorTypes->append(STARTD_SLOT_ADTYPE);
+		std::string tmp;
+		if (param(tmp, "CONDOR_VIEW_CLASSAD_TYPES")) {
+			viewCollectorTypes = split(tmp);
+			if (contains_anycase(viewCollectorTypes, STARTD_OLD_ADTYPE) &&
+				! contains_anycase(viewCollectorTypes, STARTD_SLOT_ADTYPE)) {
+				viewCollectorTypes.emplace_back(STARTD_SLOT_ADTYPE);
 			}
-			auto_free_ptr types(viewCollectorTypes->print_to_string());
+			std::string types = join(viewCollectorTypes, ",");
 			dprintf(D_ALWAYS, "CONDOR_VIEW_CLASSAD_TYPES configured, will forward ad types: %s\n",
-				types ? types.ptr() : "");
+				types.c_str());
 		}
 
 		vc_projection.set(param("COLLECTOR_FORWARD_PROJECTION"));
@@ -2517,14 +2534,14 @@ CollectorDaemon::forward_classad_to_view_collector(int cmd,
 		return;
 	}
 
-	if (filterAttr && viewCollectorTypes) {
+	if (filterAttr && !viewCollectorTypes.empty()) {
 		std::string type;
 		if (!theAd->EvaluateAttrString(std::string(filterAttr), type)) {
 			dprintf(D_ALWAYS, "Failed to lookup %s on ad, not forwarding\n", filterAttr);
 			return;
 		}
 
-		if (!viewCollectorTypes->contains_anycase(type.c_str())) {
+		if (!contains_anycase(viewCollectorTypes, type)) {
 			return;
 		}
 		dprintf(D_ALWAYS, "Forwarding ad: type=%s command=%s\n",
