@@ -204,7 +204,7 @@ class CollectorEngine : public Service
 	void  housekeeper ( int timerID = -1 );
 	int  housekeeperTimerID;
 	void cleanHashTable (CollectorHashTable &, time_t, HashFunc) const;
-	CollectorRecord* updateClassAd(CollectorHashTable&,const char*, const char *,
+	CollectorRecord* updateClassAd(CollectorHashTable&,const char*, const char *, bool,
 						   ClassAd*,AdNameHashKey&, const std::string &, int &,
 						   const condor_sockaddr& );
 
@@ -232,7 +232,7 @@ class CollectorEngine : public Service
 	ClassAd *m_collector_requirements;
 
 	bool m_forwardFilteringEnabled;
-	StringList m_forwardWatchList;
+	std::vector<std::string> m_forwardWatchList;
 	int m_forwardInterval;
 public: // so that the config code can set it.
 	bool m_allowOnlyOneNegotiator; // prior to 8.5.8, this was hard-coded to be true.
@@ -240,6 +240,6 @@ public: // so that the config code can set it.
 };
 
 AdTypes AdTypeStringToWhichAds(const char* target_type);
-AdTypes get_real_startd_ad_type(const char * mytype);
+AdTypes get_realish_startd_adtype(const char * mytype);
 
 #endif // __COLLECTOR_ENGINE_H__
