@@ -2330,10 +2330,10 @@ ClassAd * DCSchedd::addUsers(
 ClassAd * DCSchedd::enableUsers(
 	const char * usernames[], // owner@uid_domain, owner@ntdomain for windows
 	int num_usernames,
-	bool create_if,           // true if we want to create users that don't already exist
+	bool /*create_if*/,           // true if we want to create users that don't already exist
 	CondorError *errstack)
 {
-	int connect_timeout = 20;
+	const int connect_timeout = 20;
 	return actOnUsers (ENABLE_USERREC, nullptr, usernames, num_usernames, false, nullptr, errstack, connect_timeout);
 }
 
@@ -2393,8 +2393,8 @@ ClassAd * DCSchedd::disableUsers(
 }
 
 ClassAd * DCSchedd::updateUserAds(
-	ClassAdList & user_ads,	 // ads must have ATTR_USER attribute at a minimum
-	CondorError *errstack)
+	ClassAdList & /*user_ads*/,	 // ads must have ATTR_USER attribute at a minimum
+	CondorError * /*errstack*/)
 {
 	// TODO: write this
 	return nullptr; 
