@@ -22,7 +22,6 @@
 #define JOB_H
 
 #include "condor_common.h"      /* for <stdio.h> */
-#include "simplelist.h"         /* from condor_utils/ directory */
 #include "condor_id.h"
 #include "throttle_by_category.h"
 #include "read_multiple_logs.h"
@@ -180,8 +179,7 @@ class Job {
 	//remove event. Otherwise if queued nodes is 0 then all job procs are done
 	inline bool AllProcsDone() const { return !is_factory && _queuedNodeJobProcs == 0; }
 
-	bool AddScript( ScriptType script_type, const char *cmd, int defer_status,
-				time_t defer_time, std::string &whynot );
+	bool AddScript(Script* script);
 	bool AddPreSkip( int exitCode, std::string &whynot );
 
 	void SetType( NodeType type ) { _type = type; }
