@@ -294,7 +294,7 @@ bool parse(Dag *dag, const char *filename, bool useDagDir,
 					}
 					dag->SubmitDescriptions.insert(std::make_pair(std::string(nodename.c_str()), new SubmitHash()));
 					SubmitHash* submitDesc = dag->SubmitDescriptions.at(std::string(nodename.c_str()));
-					submitDesc->init();
+					submitDesc->init(JSM_DAGMAN);
 					submitDesc->setDisableFileChecks(true);
 					std::string errmsg;
 					char * stopline = NULL;
@@ -354,7 +354,7 @@ bool parse(Dag *dag, const char *filename, bool useDagDir,
 					}
 					dag->SubmitDescriptions.insert(std::make_pair(std::string(nodename.c_str()), new SubmitHash()));
 					SubmitHash* submitDesc = dag->SubmitDescriptions.at(std::string(nodename.c_str()));
-					submitDesc->init();
+					submitDesc->init(JSM_DAGMAN);
 					submitDesc->setDisableFileChecks(true);
 					std::string errmsg;
 					char * stopline = NULL;
@@ -431,7 +431,7 @@ bool parse(Dag *dag, const char *filename, bool useDagDir,
 					}
 					dag->SubmitDescriptions.insert(std::make_pair(std::string(descName.c_str()), new SubmitHash()));
 					SubmitHash* submitDesc = dag->SubmitDescriptions.at(descName.c_str());
-					submitDesc->init();
+					submitDesc->init(JSM_DAGMAN);
 					submitDesc->setDisableFileChecks(true);
 					std::string errmsg;
 					char * stopline = NULL;
@@ -1015,16 +1015,10 @@ parse_node( Dag *dag, const char * nodeName, const char * submitFileOrSubmitDesc
 			submitFileOrSubmitDesc = dagSubmitFile.c_str();
 
 		} else if ( strstr( submitFileOrSubmitDesc, DAG_SUBMIT_FILE_SUFFIX) ) {
-				// If the submit file name ends in ".condor.sub", we assume
-				// that this node is a nested DAG, and set the DAG filename
-				// accordingly.
-			nestedDagFile = submitFileOrSubmitDesc;
-			nestedDagFile.replace( nestedDagFile.find(DAG_SUBMIT_FILE_SUFFIX), 
-				sizeof(DAG_SUBMIT_FILE_SUFFIX) - 1, "" );
-			debug_printf( DEBUG_NORMAL, "Warning: the use of the JOB "
-						"keyword for nested DAGs is deprecated; please "
-						"use SUBDAG EXTERNAL instead\n" );
-			check_warning_strictness( DAG_STRICT_3 );
+			// Assume JOB commands submit file ending with '.condor.sub' is a DAG
+			// submit file. This is no longer allowed for subdags
+			debug_printf(DEBUG_NORMAL, "Error: The use of the JOB keyword for nested DAGs is prohibited.\n");
+			return false;
 		}
 	}
 
@@ -1469,6 +1463,7 @@ parse_parent(
 				" node \"%s\" and child nodes : %s\n",
 				filename, lineNumber, parent_type,
 				parent->GetJobName(), failReason.c_str());
+			return false;
 		}
 	}
 	return true;
