@@ -262,7 +262,7 @@ MultiFileCurlPlugin::InitializeCurlHandle(const std::string &url, const std::str
 		}
         r = curl_easy_setopt( _handle, CURLOPT_HEADERFUNCTION, &HeaderCallback );
 		if (r != CURLE_OK) {
-			fprintf(stderr, "Can't setopt HEADERFUNCTOIN\n");
+			fprintf(stderr, "Can't setopt HEADERFUNCTION\n");
 		}
 
         GetToken(cred, token);
@@ -516,9 +516,11 @@ MultiFileCurlPlugin::DownloadFile( const std::string &url, const std::string &lo
 	if (r != CURLE_OK) {
 		fprintf(stderr, "Can't setopt CURLOPT_WRITEDATA\n");
 	}
-    r = curl_easy_setopt( _handle, CURLOPT_HEADERDATA, _this_file_stats.get() );
-	if (r != CURLE_OK) {
-		fprintf(stderr, "Can't setopt CURLOPT_HEADERDATA\n");
+	if (!url.starts_with("ftp://")) {
+		r = curl_easy_setopt( _handle, CURLOPT_HEADERDATA, _this_file_stats.get() );
+		if (r != CURLE_OK) {
+			fprintf(stderr, "Can't setopt CURLOPT_HEADERDATA\n");
+		}
 	}
 
     if (header_list) {
@@ -1037,6 +1039,7 @@ main( int argc, char **argv ) {
                 "MultipleFileSupport = true\n"
                 "PluginVersion = \"0.2\"\n"
                 "PluginType = \"FileTransfer\"\n"
+                "ProtocolVersion = 2\n"
             );
             printf( "SupportedMethods = \"%s\"\n", SupportedMethods );
 
