@@ -185,6 +185,7 @@
 #define ATTR_DOCKER_IMAGE "DockerImage"
 #define ATTR_DOCKER_NETWORKS "DockerNetworks"
 #define ATTR_DOCKER_NETWORK_TYPE "DockerNetworkType"
+#define ATTR_DOCKER_OVERRIDE_ENTRYPOINT  "DockerOverrideEntrypoint"
 #define ATTR_DOCKER_PULL_POLICY "DockerPullPolicy"
 #define ATTR_DOCKER_VOLUMES "DockerVolumes"
 #define ATTR_DOCKER_VERSION  "DockerVersion"
@@ -264,6 +265,7 @@
 #define ATTR_HAS_CONTAINER  "HasContainer"
 #define ATTR_HAS_DOCKER  "HasDocker"
 #define ATTR_HAS_DOCKER_URL  "HasDockerURL"
+#define ATTR_HAS_START_DAEMON_AD  "HasStartDaemonAd"
 #define ATTR_HAS_SANDBOX_IMAGE  "HasSandboxImage"
 #define ATTR_HAS_SIF  "HasSIF"
 #define ATTR_HAS_SINGULARITY_PIDNAMESPACES  "HasPidNamespaces"
@@ -328,6 +330,7 @@
 #define ATTR_JOB_ACTIVATION_TEARDOWN_DURATION  "ActivationTeardownDuration"
 #define ATTR_JOB_ALLOWED_JOB_DURATION  "AllowedJobDuration"
 #define ATTR_JOB_ALLOWED_EXECUTE_DURATION "AllowedExecuteDuration"
+#define ATTR_JOB_EPOCH_WRITE_DATE "EpochWriteDate"
 #define ATTR_JOB_MATERIALIZE_DIGEST_FILE  "JobMaterializeDigestFile"
 #define ATTR_JOB_MATERIALIZE_ITEMS_FILE   "JobMaterializeItemsFile"
 #define ATTR_JOB_MATERIALIZE_LIMIT        "JobMaterializeLimit"
@@ -343,6 +346,7 @@
 #define ATTR_JOB_CHECKPOINT_NUMBER  "CheckpointNumber"
 #define ATTR_JOB_CHECKPOINT_DESTINATION  "CheckpointDestination"
 #define ATTR_JOB_CMD  "Cmd"
+#define ATTR_JOB_ORIG_CMD  "OrigCmd"
 #define ATTR_JOB_DESCRIPTION "JobDescription"
 #define ATTR_STACK_SIZE  "StackSize"
 #define ATTR_JOB_CMD_HASH  "CmdHash"
@@ -395,7 +399,6 @@
 #define ATTR_JOB_MANAGED_MANAGER  "ManagedManager"
 #define ATTR_JOB_MANIFEST_DESIRED "ManifestDesired"
 #define ATTR_JOB_MANIFEST_DIR "ManifestDir"
-#define ATTR_JOB_MATCHED  "Matched"
 #define ATTR_JOB_NEWLY_COMMITTED_TIME "NewlyCommittedTime"
 #define ATTR_JOB_NONESSENTIAL  "Nonessential"
 #define ATTR_JOB_NOOP  "IsNoopJob"
@@ -431,6 +434,7 @@
 #define ATTR_JOB_SANDBOX_JOBAD  "DropJobAdInSandbox"
 #define ATTR_JOB_SET_ID "JobSetId"
 #define ATTR_JOB_SET_NAME "JobSetName"
+#define ATTR_JOB_SUBMIT_FILE  "JobSubmitFile"
 #define ATTR_JOB_SUBMISSION  "Submission"
 #define ATTR_JOB_SUBMISSION_ID  "SubmissionId"
 #define ATTR_JOB_BATCH_ID "JobBatchId"
@@ -449,6 +453,8 @@
 #define ATTR_JOB_WALL_CLOCK_CKPT  "WallClockCheckpoint"
 #define ATTR_JOB_QUEUE_BIRTHDATE  "JobQueueBirthdate"
 #define ATTR_JOB_REQUIRES_SANDBOX  "JobRequiresSandbox"
+#define ATTR_JOB_CURRENT_RECONNECT_ATTEMPT "JobCurrentReconnectAttempt"
+#define ATTR_TOTAL_JOB_RECONNECT_ATTEMPTS "TotalJobReconnectAttempts"
 #define ATTR_JOB_VM_TYPE  "JobVMType"
 #define ATTR_JOB_VM_MEMORY  "JobVMMemory"
 #define ATTR_JOB_VM_VCPUS  "JobVM_VCPUS"
@@ -872,6 +878,7 @@
 #define ATTR_CONDOR_VERSION "CondorVersion"
 #define ATTR_CHECKPOINT_FILES "TransferCheckpoint"
 #define ATTR_PRESERVE_RELATIVE_PATHS "PreserveRelativePaths"
+#define ATTR_FAILURE_FILES "FailureFiles"
 
 #define ATTR_TRANSFER_SOCKET  "TransferSocket"
 #define ATTR_SERVER_TIME  "ServerTime"
@@ -1054,6 +1061,11 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_REQUEST_GPUS  "RequestGPUs"
 #define ATTR_REQUIRE_GPUS  "RequireGPUs"
 #define ATTR_REQUEST_VIRTUAL_MEMORY  "RequestVirtualMemory"
+// GPUs property constraints
+#define ATTR_GPUS_MIN_MEMORY "GPUsMinMemory"
+#define ATTR_GPUS_MIN_CAPABILITY "GPUsMinCapability"
+#define ATTR_GPUS_MAX_CAPABILITY "GPUsMaxCapability"
+#define ATTR_GPUS_MIN_RUNTIME "GPUsMinRuntime"
 
 // machine resource prefixes
 #define ATTR_REQUEST_PREFIX  "Request"
@@ -1068,14 +1080,6 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_CLAIM_ID_LIST "ClaimIdList"
 #define ATTR_CONSUMPTION_PREFIX "Consumption"
 
-// This is a record of the job exit status from a standard universe job exit
-// via waitpid. It is in the job ad to implement the terminate_pending
-// feature. It has to be here because of rampant global variable usage in the
-// standard universe shadow. It saved a tremendous amount of code to just
-// put this value in the job ad.
-#define ATTR_WAITPID_STATUS  "WaitpidStatus"
-#define ATTR_TERMINATION_REASON  "TerminationReason"
-
 // Valid settings for ATTR_JOB_MANAGED.
 	// Managed by an external process (gridmanager)
 #define MANAGED_EXTERNAL  "External"
