@@ -161,7 +161,8 @@ void dprintf ( int flags, DPF_IDENT ident, const char *fmt, ... ) CHECK_PRINTF_F
 int dprintf_config( 
 	const char *subsys,  // in: subsystem name to use for param lookups
 	struct dprintf_output_settings *p_info = NULL, // in,out: if != NULL results of config parsing returned here
-	int c_info = 0); // in: number of entries in p_info array on input.                  
+	int c_info = 0,
+	const char* log2arg = nullptr); // in: number of entries in p_info array on input.
 
 int dprintf_config_tool(const char* subsys, const char * flags, const char * logfile = nullptr);
 int dprintf_config_tool_on_error(const char * flags);
@@ -184,6 +185,11 @@ void _condor_parse_merge_debug_flags(
 	DebugOutputChoice & basic, // in,out: basic output choice
 	DebugOutputChoice & verbose); // in,out: verbose output choice, expect this to get folded into basic someday.
 
+// parse a string containing dprintf category and optional verbosity e.g  D_SECURITY:2
+// returns true if the string had at least one category, and returns the cat and verbosity value as cat_and_verb
+// if a hdr_flags pointer is passed, any header flag bits like D_SUB_SECOND are returned there
+bool parse_debug_cat_and_verbosity(const char * strFlags, int & cat_and_verb, unsigned int * hdr_flags=nullptr);
+
 bool dprintf_to_term_check();
 
 void _condor_dprintf_va ( int flags, DPF_IDENT ident, const char* fmt, va_list args );
@@ -231,6 +237,15 @@ void dprintf_dump_stack(void);
  */
 void dprintf_pause_buffering();
 
+/* open any logs in the given directory as whatever priv is currently set
+*  caller is responsible for init_user_ids() and set_user_priv() if user_priv is desired
+*/
+int dprintf_open_logs_in_directory(const char * dir, bool fTruncate=false);
+
+/* close any dprintf logs in the given directory
+*/
+int dprintf_close_logs_in_directory(const char * dir, bool permanent=true);
+
 time_t dprintf_last_modification(void);
 void dprintf_touch_log(void);
 /* write dprintf contribution to the daemon header */
