@@ -1293,7 +1293,8 @@ Stream::get_secret( std::string& s )
 
 	retval = get_string_ptr(str, len);
 	if (retval) {
-		s.assign(str ? str : "", len);
+		// len includes a NUL terminator, don't make that part of the string
+		s.assign(str ? str : "", len-1);
 	}
 
 	restore_crypto_after_secret();
