@@ -702,10 +702,6 @@ class Scheduler : public Service
 	void			mail_problem_message();
 	bool            FindRunnableJobForClaim(match_rec* mrec);
 
-		// hashtable used to hold matching ClassAds for Globus Universe
-		// jobs which desire matchmaking.
-	HashTable <PROC_ID, ClassAd *> *resourcesByProcID;
-
 	bool usesLocalStartd() const { return m_use_startd_for_local;}
 
 	//
@@ -760,6 +756,14 @@ class Scheduler : public Service
 	// Return a pointer to the protected URL map for late materilization factories
 	MapFile* getProtectedUrlMap() { return &m_protected_url_map; }
 
+	ClassAd *getLocalStarterAd() {
+		static bool firstTime = true;
+		if (firstTime) {
+			firstTime = false;
+			m_local_starter_ad.Assign(ATTR_CONDOR_SCRATCH_DIR, "#CoNdOrScRaTcHdIr#");
+		}
+		return &m_local_starter_ad;
+	}
 private:
 
 	bool JobCanFlock(classad::ClassAd &job_ad, const std::string &pool);
@@ -797,6 +801,7 @@ private:
 	ClassAd             m_userRecDefaultsAd;
 	ClassAd             m_extendedSubmitCommands;
 	std::string         m_extendedSubmitHelpFile;
+	ClassAd             m_local_starter_ad;
 
 	// information about the command port which Shadows use
 	char*			MyShadowSockName;
