@@ -45,7 +45,6 @@ ParallelShadow::~ParallelShadow() {
     for ( size_t i=0 ; i<ResourceList.size() ; i++ ) {
         delete ResourceList[i];
     }
-	daemonCore->Cancel_Command( SHADOW_UPDATEINFO );
 }
 
 void 
@@ -59,16 +58,6 @@ ParallelShadow::init( ClassAd* job_ad, const char* schedd_addr, const char *xfer
         // BaseShadow::baseInit - basic init stuff...
     baseInit( job_ad, schedd_addr, xfer_queue_contact_info );
 
-		// Register command which gets updates from the starter
-		// on the job's image size, cpu usage, etc.  Each kind of
-		// shadow implements it's own version of this to deal w/ it
-		// properly depending on parallel vs. serial jobs, etc. 
-	daemonCore->
-		Register_Command( SHADOW_UPDATEINFO, "SHADOW_UPDATEINFO", 
-						  (CommandHandlercpp)&ParallelShadow::updateFromStarter,
-						  "ParallelShadow::updateFromStarter", this, DAEMON ); 
-
-
         // make first remote resource the "master".  Put it first in list.
     MpiResource *rr = new MpiResource( this );
 	parallelMasterResource = rr;
@@ -316,7 +305,13 @@ ParallelShadow::getResources( int /* timerID */ )
 				delete sock;
 				return;
 			};
-			ASSERT( jobAdNumInProc == numInProc);
+			if (jobAdNumInProc != numInProc) {
+				dprintf(D_ALWAYS, "ERROR -- job needs %d slots, but schedd gave us %d slots -- giving up\n", jobAdNumInProc, numInProc);
+				BaseShadow::shutDown(JOB_NOT_STARTED);
+				sock->end_of_message();
+				delete sock;
+				return;
+			}
         } // end of for loop for this proc
 
     } // end of for loop on all procs...
@@ -631,21 +626,6 @@ ParallelShadow::replaceNode ( ClassAd *ad, int nodenum ) {
 }
 
 
-int
-ParallelShadow::updateFromStarter(int  /*command*/, Stream *s)
-{
-	ClassAd update_ad;
-	s->decode();
-	if (!getClassAd(s, update_ad)) {
-		s->end_of_message();
-		dprintf(D_ALWAYS, "ParallelShadow::updateFromStarter could not get Class Ad\n");
-		return FALSE;
-	}
-	s->end_of_message();
-	fix_update_ad(update_ad);
-	return updateFromStarterClassAd(&update_ad);
-}
-
 int
 ParallelShadow::updateFromStarterClassAd(ClassAd* update_ad)
 {
