@@ -206,6 +206,9 @@ RemoteResource::activateClaim( int starterVersion )
 				   (SocketHandlercpp)&RemoteResource::handleSysCalls,
 				   "HandleSyscalls", this );
 			setResourceState( RR_STARTUP );
+
+			// clear out reconnect attempt
+			jobAd->AssignExpr(ATTR_JOB_CURRENT_RECONNECT_ATTEMPT, "undefined");
 			hadContact();
 
 				// This expiration time we calculate here may be
@@ -1771,14 +1774,17 @@ RemoteResource::resourceExit( int reason_for_exit, int exit_status )
 
 	// Record the activation stop time (HTCONDOR-861) and set the
 	// corresponding duration attributes.
-	activation.TerminationTime = time(NULL);
-	time_t ActivationDuration = activation.TerminationTime - activation.StartTime;
-	time_t ActivationTeardownDuration = activation.TerminationTime - activation.ExitExecutionTime;
+	if (began_execution) {
+		// but only if we have begun execution, which is where we set activation.StartTime
+		activation.TerminationTime = time(nullptr);
+		time_t ActivationDuration = activation.TerminationTime - activation.StartTime;
+		time_t ActivationTeardownDuration = activation.TerminationTime - activation.ExitExecutionTime;
 
-	// Where would these attributes get rotated?  Here?
-	jobAd->InsertAttr( ATTR_JOB_ACTIVATION_DURATION, ActivationDuration );
-	jobAd->InsertAttr( ATTR_JOB_ACTIVATION_TEARDOWN_DURATION, ActivationTeardownDuration );
-	shadow->updateJobInQueue( U_STATUS );
+		// Where would these attributes get rotated?  Here?
+		jobAd->InsertAttr( ATTR_JOB_ACTIVATION_DURATION, ActivationDuration );
+		jobAd->InsertAttr( ATTR_JOB_ACTIVATION_TEARDOWN_DURATION, ActivationTeardownDuration );
+		shadow->updateJobInQueue( U_STATUS );
+	}
 
 	// record the start time of transfer output into the job ad.
 	time_t tStart = -1, tEnd = -1;
@@ -1791,17 +1797,6 @@ RemoteResource::resourceExit( int reason_for_exit, int exit_status )
 		jobAd->Assign(ATTR_JOB_CURRENT_FINISH_TRANSFER_INPUT_DATE, (int)tEnd);
 	}
 
-#if 0 // tj: this seems to record only transfer output time, turn it off for now.
-	FileTransfer::FileTransferInfo fi = filetrans.GetInfo();
-	if (fi.duration) {
-		float cumulativeDuration = 0.0;
-		if ( ! jobAd->LookupFloat(ATTR_CUMULATIVE_TRANSFER_TIME, cumulativeDuration)) { 
-			cumulativeDuration = 0.0; 
-		}
-		jobAd->Assign(ATTR_CUMULATIVE_TRANSFER_TIME, cumulativeDuration + fi.duration );
-	}
-#endif
-
 	if( exit_value == -1 ) {
 			/* 
 			   Backwards compatibility code...  If we don't have a
@@ -2034,6 +2029,12 @@ RemoteResource::attemptReconnect( int /* timerID */ )
 		// if if this attempt fails, we need to remember we tried
 	reconnect_attempts++;
 
+	jobAd->Assign(ATTR_JOB_CURRENT_RECONNECT_ATTEMPT, reconnect_attempts);
+	int total_reconnects = 0;
+	jobAd->LookupInteger(ATTR_TOTAL_JOB_RECONNECT_ATTEMPTS, total_reconnects);
+	total_reconnects++;
+	jobAd->Assign(ATTR_TOTAL_JOB_RECONNECT_ATTEMPTS, total_reconnects);
+	shadow->updateJobInQueue(U_STATUS);
 	
 		// ask the startd if the starter is still alive, and if so,
 		// where it is located.  note we must ask the startd, even
@@ -2268,6 +2269,8 @@ RemoteResource::initFileTransfer()
 	const char * currentFile = NULL;
 	Directory spoolDirectory( spoolPath.c_str() );
 	while( (currentFile = spoolDirectory.Next()) ) {
+		// getNumberFromFileName() ignores FAILURE files, which is
+		// exactly the behavior we want here.
 		int manifestNumber = manifest::getNumberFromFileName( currentFile );
 		if( manifestNumber > largestManifestNumber ) {
 			largestManifestNumber = manifestNumber;
@@ -2494,6 +2497,9 @@ RemoteResource::requestReconnect( void )
 		}
 	}
 
+	jobAd->AssignExpr(ATTR_JOB_CURRENT_RECONNECT_ATTEMPT, "undefined");
+	shadow->updateJobInQueue(U_STATUS);
+
 	reconnect_attempts = 0;
 	hadContact();
 
