@@ -78,6 +78,9 @@ public:
 	stats_entry_recent<int>	total_rank_preemptions;
 	stats_entry_recent<int>	total_user_prio_preemptions;
 	stats_entry_recent<int>	total_job_starts;
+	stats_entry_recent<int>	total_claim_requests;
+	stats_entry_recent<int>	total_activation_requests;
+	stats_entry_recent<int> total_new_dslot_unwilling;
 	stats_entry_recent<Probe> job_busy_time;
 	stats_entry_recent<Probe> job_duration;
 
@@ -97,6 +100,9 @@ public:
 		pool.AddProbe("JobRankPreemptions", &total_rank_preemptions);
 		pool.AddProbe("JobUserPrioPreemptions", &total_user_prio_preemptions);
 		pool.AddProbe("JobStarts", &total_job_starts);
+		pool.AddProbe("ClaimRequests", &total_claim_requests);
+		pool.AddProbe("ActivationRequests", &total_activation_requests);
+		pool.AddProbe("NewDSlotNotMatch", &total_new_dslot_unwilling);
 
 		// publish two Miron probes, showing only XXXCount if count is zero, and
 		// also XXXMin, XXXMax and XXXAvg if count is non-zero
@@ -287,6 +293,7 @@ public:
 	void		init_config_classad( void );
 	void		updateExtrasClassAd( ClassAd * cap );
 	void		publish_daemon_ad(ClassAd & ad);
+	void		final_update_daemon_ad();
 
 	void		addResource( Resource* );
 	bool		removeResource( Resource* );
@@ -408,9 +415,13 @@ public:
 	}
 
 	bool compute_resource_conflicts();
-	const ResBag & resourcesInUse() { return primary_res_in_use; }
-	const ResBag & bk_resourcesInUse() { return backfill_res_in_use; }
 	void printSlotAds(const char * slot_types) const;
+	void refresh_classad_resources(Resource * rip) const {
+		rip->refresh_classad_resources(primary_res_in_use);
+	}
+	void publish_static_slot_resources(Resource * rip, classad::ClassAd * cad) const {
+		rip->publish_static_resources(cad, primary_res_in_use);
+	}
 
 	template <typename Func>
 	void walk(Func fn) {
@@ -440,6 +451,7 @@ private:
 	// backfill slot advertising code
 	ResBag primary_res_in_use;
 	ResBag backfill_res_in_use;
+	ResBag excess_backfill_res; // difference between size of backfill and primary bag when both are fully idle
 
 	IdDispenser* id_disp;
 	bool 		is_shutting_down;
@@ -464,6 +476,8 @@ private:
 	std::vector<Resource*>	_pending_removes;
 	void _remove_and_delete_slot_res(Resource*);
 	void _complete_removes();
+	// called in init_resources after the configured slots have been created 
+	void _post_init_resources();
 
 	// search helper templates
 	template <typename Ret, typename Filter>
@@ -517,10 +531,11 @@ private:
 	std::string m_token_request_id;
 	std::string m_token_client_id;
 	Daemon *m_token_daemon{nullptr};
+	DCTokenRequester m_token_requester;
 
 #if HAVE_BACKFILL
-	bool backfillConfig( void );
-	bool m_backfill_shutdown_pending;
+	bool backfillMgrConfig( void );
+	bool m_backfill_mgr_shutdown_pending;
 #endif /* HAVE_BACKFILL */
 
 #if HAVE_JOB_HOOKS
@@ -560,8 +575,9 @@ private:
 	std::unique_ptr<VolumeManager> m_volume_mgr;
 #endif
 
-	DCTokenRequester m_token_requester;
+#ifdef HAVE_DATA_REUSE_DIR
 	std::unique_ptr<htcondor::DataReuseDirectory> m_reuse_dir;
+#endif
 };
 
 
