@@ -60,8 +60,8 @@ int	update_interval = 0;	// Interval to update CM
 int enable_single_startd_daemon_ad = 0;
 
 // String Lists
-StringList *startd_job_attrs = NULL;
-StringList *startd_slot_attrs = NULL;
+std::vector<std::string> startd_job_attrs;
+std::vector<std::string> startd_slot_attrs;
 static StringList *valid_cod_users = NULL; 
 
 // Hosts
@@ -505,6 +505,8 @@ init_params( int first_time)
 			enable_single_startd_daemon_ad = 2;
 		}
 	}
+	dprintf(D_STATUS, "ENABLE_STARTD_DAEMON_AD=%d (%s)\n", enable_single_startd_daemon_ad,
+		send_daemon_ad.ptr() ? send_daemon_ad.ptr() : "");
 
 	if( accountant_host ) {
 		free( accountant_host );
@@ -523,57 +525,24 @@ init_params( int first_time)
 
 	// Fill in *_JOB_ATTRS
 	//
-	if (startd_job_attrs) {
-		startd_job_attrs->clearAll();
-	} else {
-		startd_job_attrs = new StringList();
-	}
-	param_and_insert_unique_items("STARTD_JOB_ATTRS", *startd_job_attrs);
+	startd_job_attrs.clear();
+	param_and_insert_unique_items("STARTD_JOB_ATTRS", startd_job_attrs);
 	// merge in the deprecated _EXPRS config
-	param_and_insert_unique_items("STARTD_JOB_EXPRS", *startd_job_attrs);
+	param_and_insert_unique_items("STARTD_JOB_EXPRS", startd_job_attrs);
 	// Now merge in the attrs required by HTCondor - this knob is a secret from users
-	param_and_insert_unique_items("SYSTEM_STARTD_JOB_ATTRS", *startd_job_attrs);
-	if (startd_job_attrs->isEmpty()) {
-		delete startd_job_attrs;
-		startd_job_attrs = NULL;
-	}
+	param_and_insert_unique_items("SYSTEM_STARTD_JOB_ATTRS", startd_job_attrs);
 
 	// Fill in *_SLOT_ATTRS
 	//
-	if (startd_slot_attrs) {
-		startd_slot_attrs->clearAll();
-	} else {
-		startd_slot_attrs = new StringList();
-	}
-	param_and_insert_unique_items("STARTD_SLOT_ATTRS", *startd_slot_attrs);
-	param_and_insert_unique_items("STARTD_SLOT_EXPRS", *startd_slot_attrs);
+	startd_slot_attrs.clear();
+	param_and_insert_unique_items("STARTD_SLOT_ATTRS", startd_slot_attrs);
+	param_and_insert_unique_items("STARTD_SLOT_EXPRS", startd_slot_attrs);
 
 	// now insert attributes needed by HTCondor
-	param_and_insert_unique_items("SYSTEM_STARTD_SLOT_ATTRS", *startd_slot_attrs);
-	if (startd_slot_attrs->isEmpty()) {
-		delete  startd_slot_attrs;
-		startd_slot_attrs = NULL;
-	}
-
-	console_slots = param_integer( "SLOTS_CONNECTED_TO_CONSOLE", -12345);
-	if (console_slots == -12345) {
-		// if no value set, try the old names...
-		console_slots = resmgr->m_attr->num_cpus();
-		console_slots = param_integer( "VIRTUAL_MACHINES_CONNECTED_TO_CONSOLE",
-		                param_integer( "CONSOLE_VMS",
-		                param_integer( "CONSOLE_CPUS",
-		                console_slots)));
-	}
-
-	keyboard_slots = param_integer( "SLOTS_CONNECTED_TO_KEYBOARD", -12345);
-	if (keyboard_slots == -12345) {
-		// if no value set, try the old names...
-		keyboard_slots = resmgr->m_attr->num_cpus();
-		keyboard_slots = param_integer( "VIRTUAL_MACHINES_CONNECTED_TO_KEYBOARD",
-		                 param_integer( "KEYBOARD_VMS",
-		                 param_integer( "KEYBOARD_CPUS", 1)));
-	}
+	param_and_insert_unique_items("SYSTEM_STARTD_SLOT_ATTRS", startd_slot_attrs);
 
+	console_slots = param_integer( "SLOTS_CONNECTED_TO_CONSOLE", 0);
+	keyboard_slots = param_integer( "SLOTS_CONNECTED_TO_KEYBOARD", 0);
 	disconnected_keyboard_boost = param_integer( "DISCONNECTED_KEYBOARD_IDLE_BOOST", 1200 );
 
 	startd_noclaim_shutdown = param_integer( "STARTD_NOCLAIM_SHUTDOWN", 0 );
