@@ -164,6 +164,7 @@ public:
 	void notifyJobPreSpawn( void );
 
 	void notifyExecutionExit( void );
+	void notifyGenericEvent( const ClassAd & event );
 
 		/** Notify the shadow that the job exited. This will not only
 			update the job ad with the termination information of the job,
@@ -205,11 +206,9 @@ public:
 		   Send a periodic update ClassAd to the shadow.
 
 		   @param update_ad Update ad to use if you've already got the info
-		   @param insure_update Should we insure the update gets there?
 		   @return true if success, false if failure
 		*/
-	virtual bool periodicJobUpdate(ClassAd* update_ad = NULL,
-								   bool insure_update = false);
+	virtual bool periodicJobUpdate(ClassAd* update_ad = NULL);
 
 
 		// // // // // // // // // // // //
@@ -278,14 +277,11 @@ private:
 	// Should only be called from publish[Update|JobExit]Ad().
 	bool publishStartdUpdates( ClassAd* ad );
 
-		/** Send an update ClassAd to the shadow.  The "insure_update"
-			just means do we make sure the update gets there.  It has
-			nothing to do with the "insure" memory analysis tool.
+		/** Send an update ClassAd to the shadow.
 			@param update_ad Update ad to use if you've already got the info 
-			@param insure_update Should we insure the update gets there?
 			@return true if success, false if failure
 		*/
-	bool updateShadow( ClassAd* update_ad, bool insure_update = false );
+	bool updateShadow( ClassAd* update_ad );
 
 		/** Send an update ClassAd to the startd.
 			@param ad Update ad
