@@ -35,9 +35,11 @@
 #include "profile.WINDOWS.h" // for OwnerProfile class
 #endif
 
+#if 1 //def HAVE_DATA_REUSE_DIR TOO: remove this someday
 namespace htcondor {
 class DataReuseDirectory;
 }
+#endif
 
 /** The starter class.  Basically, this class does some initialization
 	stuff and manages a set of UserProc instances, each of which 
@@ -318,7 +320,11 @@ public:
 	int GetShutdownExitCode() const { return m_shutdown_exit_code; };
 	void SetShutdownExitCode( int code ) { m_shutdown_exit_code = code; };
 
+#ifdef HAVE_DATA_REUSE_DIR
 	htcondor::DataReuseDirectory * getDataReuseDirectory() const {return m_reuse_dir.get();}
+#else
+	htcondor::DataReuseDirectory * getDataReuseDirectory() const {return nullptr;}
+#endif
 
 	void SetJobEnvironmentReady(const bool isReady) {m_job_environment_is_ready = isReady;}
 
@@ -376,12 +382,6 @@ private:
 
 	bool WriteAdFiles() const;
 
-		//
-		// Check the disk usage of this job; if over the limits, potentially
-		// put the job on hold.
-		//
-	void CheckDiskUsage( int timerID = -1 );
-
 		// // // // // // // //
 		// Private Data Members
 		// // // // // // // //
@@ -424,13 +424,11 @@ private:
 		// the maximum permitted disk usage and the polling timer
 		//
 #if defined(LINUX)
-	int64_t m_lvm_max_size_kb{0};
-	int m_lvm_poll_tid{-1};
+	void CheckLVUsage( int timerID = -1 );
+	int64_t m_lvm_lv_size_kb{0};
 	time_t m_lvm_last_space_issue{-1};
+	int m_lvm_poll_tid{-1};
 	bool m_lvm_held_job{false};
-	std::string m_lvm_thin_volume;
-	std::string m_lvm_thin_pool;
-	std::string m_lvm_volume_group;
 #endif
 
 	UserProc* pre_script;
@@ -451,8 +449,10 @@ private:
 		// starter's exit code be?
 	int m_shutdown_exit_code;
 
-		// Manage the data reuse directory.
+#ifdef HAVE_DATA_REUSE_DIR
+	// Manage the data reuse directory.
 	std::unique_ptr<htcondor::DataReuseDirectory> m_reuse_dir;
+#endif
 
 	// The string to set the tmp env vars to
 	std::string tmpdir;
