@@ -38,6 +38,7 @@
 #include "has_sysadmin_cap.h"
 #include "starter_util.h"
 #include "proc_family_direct_cgroup_v2.h"
+#include "nvidia_utils.h"
 #include <array>
 
 #include <sstream>
@@ -337,6 +338,17 @@ VanillaProc::StartJob()
 		}
 		fi.cgroup_cpu_shares = 100 * numCores;
 
+		if (param_boolean("STARTER_HIDE_GPU_DEVICES", true)) {
+			// Potentially disable GPU devices from job
+			std::string available_gpus;
+			const char *gpu_expr = "join(\",\",evalInEachContext(strcat(\"GPU-\",DeviceUuid),AvailableGPUs))";
+			classad::Value v;
+			Starter->jic->machClassAd()->EvaluateExpr(gpu_expr, v);
+			v.IsStringValue(available_gpus);
+
+			// will remain empty if not set, meaning hide all
+			fi.cgroup_hide_devices = nvidia_env_var_to_exclude_list(available_gpus);
+		}
 		int64_t memory = 0;
 		if (!Starter->jic->machClassAd()->LookupInteger(ATTR_MEMORY, memory)) {
 			dprintf(D_ALWAYS, "Invalid value of memory in machine ClassAd; aborting\n");
@@ -682,7 +694,6 @@ VanillaProc::StartJob()
 bool
 VanillaProc::PublishUpdateAd( ClassAd* ad )
 {
-	dprintf( D_FULLDEBUG, "In VanillaProc::PublishUpdateAd()\n" );
 	static unsigned int max_rss = 0;
 #if HAVE_PSS
 	static unsigned int max_pss = 0;
@@ -825,7 +836,22 @@ VanillaProc::notifySuccessfulPeriodicCheckpoint( int checkpointNumber ) {
 	int newlyCommittedTime = (int)timersub_double(job_exit_time, job_start_time);
 	updateAd.Assign( ATTR_JOB_NEWLY_COMMITTED_TIME, newlyCommittedTime );
 
-	Starter->jic->periodicJobUpdate( & updateAd, false );
+	Starter->jic->periodicJobUpdate( & updateAd );
+
+	// Let's not try to be subtle and confusing (by reacting to the above
+	// update in the shadow instead of to a specific event).
+	ClassAd eventAd;
+	eventAd.InsertAttr( "EventType", "SuccessfulCheckpoint" );
+	eventAd.InsertAttr( ATTR_JOB_CHECKPOINT_NUMBER, checkpointNumber );
+	Starter->jic->notifyGenericEvent( eventAd );
+}
+
+void
+VanillaProc::notifyFailedPeriodicCheckpoint( int checkpointNumber ) {
+    ClassAd ad;
+    ad.InsertAttr( "EventType", "FailedCheckpoint" );
+    ad.InsertAttr( ATTR_JOB_CHECKPOINT_NUMBER, checkpointNumber );
+    Starter->jic->notifyGenericEvent( ad );
 }
 
 void VanillaProc::recordFinalUsage() {
@@ -861,13 +887,16 @@ void VanillaProc::restartCheckpointedJob() {
 		JobAd->LookupInteger( ATTR_JOB_CHECKPOINT_NUMBER, checkpointNumber );
 	}
 
-	if( Starter->jic->uploadCheckpointFiles(checkpointNumber + 1) ) {
-			++checkpointNumber;
+	// Because not all upload attempts fail without writing any data, we
+	// need to clean up after failed attempts, which implies numbering them.
+	++checkpointNumber;
+	if( Starter->jic->uploadCheckpointFiles(checkpointNumber) ) {
 			notifySuccessfulPeriodicCheckpoint(checkpointNumber);
 	} else {
 			// We assume this is a transient failure and will try
 			// to transfer again after the next periodic checkpoint.
 			dprintf( D_ALWAYS, "Failed to transfer checkpoint.\n" );
+			notifyFailedPeriodicCheckpoint(checkpointNumber);
 	}
 
 	// While it's arguably sensible to kill the process family
@@ -900,7 +929,7 @@ VanillaProc::outOfMemoryEvent() {
 	// memory they used.
 	ClassAd updateAd;
 	PublishUpdateAd( &updateAd );
-	Starter->jic->periodicJobUpdate( &updateAd, true );
+	Starter->jic->periodicJobUpdate( &updateAd );
 	int64_t usageKB = 0;
 	// This is the peak
 	updateAd.LookupInteger(ATTR_IMAGE_SIZE, usageKB);
@@ -932,8 +961,8 @@ VanillaProc::outOfMemoryEvent() {
 
 	std::string ss;
 	if (m_memory_limit >= 0) {
-		formatstr(ss, "Job has gone over cgroup memory limit of %ld megabytes. Peak usage: %ld megabytes.  Consider resubmitting with a higher request_memory.", 
-				(m_memory_limit / (1024 * 1024)), usageMB);
+		formatstr(ss, "Job has gone over cgroup memory limit of %lld megabytes. Peak usage: %lld megabytes.  Consider resubmitting with a higher request_memory.", 
+				(long long)(m_memory_limit / (1024 * 1024)), (long long)usageMB);
 	} else {
 		ss = "Job has encountered an out-of-memory event.";
 	}
