@@ -35,7 +35,6 @@
 #include "subsystem_info.h"
 
 #include "condor_uid.h"
-#include "string_list.h"
 #include "directory.h"
 #include "condor_qmgr.h"
 #include "condor_classad.h"
@@ -51,7 +50,6 @@
 #include "file_lock.h"
 #include "filename_tools.h"
 #include "ipv6_hostname.h"
-#include "subsystem_info.h"
 #include "my_popen.h"
 #include "stl_string_utils.h"
 #include "checkpoint_cleanup_utils.h"
@@ -76,7 +74,7 @@ extern void		_condor_set_debug_flags( const char *strflags, int flags );
 // Define this to check for memory leaks
 
 char		*Spool;				// dir for condor job queue
-StringList   ExecuteDirs;		// dirs for execution of condor jobs
+std::vector<std::string> ExecuteDirs;	// dirs for execution of condor jobs
 char		*Log;				// dir for condor program logs
 char		*DaemonSockDir;     // dir for daemon named sockets
 char        *PublicFilesWebrootDir; // dir for public input file hash links
@@ -106,6 +104,7 @@ int send_email();
 bool is_valid_shared_exe( const char *name );
 bool is_ckpt_file_or_submit_digest(const char *name, JOB_ID_KEY & jid);
 bool is_ccb_file( const char *name );
+bool is_gangliad_file( const char *name );
 bool touched_recently(char const *fname,time_t delta);
 bool linked_recently(char const *fname,time_t delta);
 #ifdef HAVE_HTTP_PUBLIC_FILES
@@ -479,12 +478,18 @@ check_spool_dir()
 	}
 
 	//List of known history like config knobs to not delete if in spool
-	std::string history_knobs[] = {"HISTORY","JOB_EPOCH_HISTORY","STARTD_HISTORY", "COLLECTOR_PERSISTENT_AD_LOG"};
+	static const std::string valid_knobs[] = {
+		"HISTORY",
+		"JOB_EPOCH_HISTORY",
+		"STARTD_HISTORY",
+		"COLLECTOR_PERSISTENT_AD_LOG",
+		"LVM_BACKING_FILE",
+	};
 	//Param the knobs for the file name and add to data structure
-	std::deque<std::string> history_files;
-	for(auto &knob : history_knobs) {
+	std::deque<std::string> config_defined_files;
+	for(const auto &knob : valid_knobs) {
 		auto_free_ptr option(param(knob.c_str()));
-		if (option) { history_files.push_back(condor_basename(option)); }
+		if (option) { config_defined_files.push_back(condor_basename(option)); }
 	}
 
 	well_known_list.initializeFromString (ValidSpoolFiles);
@@ -538,16 +543,16 @@ check_spool_dir()
 			continue;
 		}
 
-		//Check to see if file is a history
-		bool isValidHistory = false;
-		for (auto &file : history_files) {
+		//Check to see if file is a valid config defined file
+		bool isValidCondorFile = false;
+		for (const auto &file : config_defined_files) {
 			if (strncmp(f, file.c_str(), file.length()) == MATCH) {
-				isValidHistory = true;
+				isValidCondorFile = true;
 				break;
 			}
 		}
-		//If we found a match to a history file then mark this as good
-		if (isValidHistory) {
+		//If we found a valid config defined file then mark this as good
+		if (isValidCondorFile) {
 			good_file( Spool, f );
 			continue;
 		}
@@ -564,6 +569,12 @@ check_spool_dir()
 			continue;
 		}
 
+			// See if it's a GangliaD reset metrics file
+		if ( is_gangliad_file( f ) ) {
+			good_file( Spool, f );
+			continue;
+		}
+
 		// if the file is a directory, look into it.
 		if (dir.IsDirectory() && ! dir.IsSymlink()) {
 
@@ -819,6 +830,19 @@ is_ccb_file( const char *name )
 	return false;
 }
 
+/*
+  Check whether the given file is a GandliaD metrics file
+*/
+bool
+is_gangliad_file( const char *name )
+{
+	if( strstr(name,".ganglia_metrics") ) {
+		return true;
+	}
+	return false;
+}
+
+
 
 /*
   Scan the execute directory looking for bogus files.
@@ -856,16 +880,14 @@ check_execute_dir()
 		return;
 	}
 
-	ExecuteDirs.rewind();
-	char const *Execute;
-	while( (Execute=ExecuteDirs.next()) ) {
-		Directory dir( Execute, PRIV_ROOT );
+	for (const auto& Execute: ExecuteDirs) {
+		Directory dir( Execute.c_str(), PRIV_ROOT );
 		while( (f = dir.Next()) ) {
 			if( busy ) {
-				good_file( Execute, f );	// let anything go here
+				good_file( Execute.c_str(), f );	// let anything go here
 			} else {
 				if( dir.GetCreateTime() < now ) {
-					bad_file( Execute, f, dir ); // junk it
+					bad_file( Execute.c_str(), f, dir ); // junk it
 				}
 				else {
 						// In case the startd just started up a job, we
@@ -874,8 +896,8 @@ check_execute_dir()
 						// to wait for the startd to restart before they
 						// are cleaned up.)
 					dprintf(D_FULLDEBUG, "In %s, found %s with recent "
-					        "creation time.  Not removing.\n", Execute, f );
-					good_file( Execute, f ); // too young to kill
+					        "creation time.  Not removing.\n", Execute.c_str(), f );
+					good_file( Execute.c_str(), f ); // too young to kill
 				}
 			}
 		}
@@ -1190,7 +1212,7 @@ init_params()
 
 	char *Execute = param("EXECUTE");
 	if( Execute ) {
-		ExecuteDirs.append(Execute);
+		ExecuteDirs.emplace_back(Execute);
 		free(Execute);
 		Execute = NULL;
 	}
@@ -1202,8 +1224,8 @@ init_params()
 		for (size_t ii = 0; ii < params.size(); ++ii) {
 			Execute = param(params[ii].c_str());
 			if (Execute) {
-				if ( ! ExecuteDirs.contains(Execute)) {
-					ExecuteDirs.append(Execute);
+				if ( ! contains(ExecuteDirs, Execute)) {
+					ExecuteDirs.emplace_back(Execute);
 				}
 				free(Execute);
 			}
