@@ -117,7 +117,9 @@ static const struct Translation MyCommandTranslation[] = {
 	{ "", 0 }
 };
 
+#if 0
 static const char * cmd_to_str(int cmd) { return getNameFromNum(cmd, MyCommandTranslation); }
+#endif
 static int str_to_cmd(const char * str) { return getNumFromName(str, MyCommandTranslation); }
 static void print_results(ClassAd * ad, const char * op);
 
@@ -395,7 +397,8 @@ main( int argc, const char *argv[] )
 			}
 			prmask.registerFormatF( argv[i+1], argv[i+2], FormatOptionNoTruncate );
 			if ( ! IsValidClassAdExpression(argv[i+2], &attrs, NULL)) {
-				return i+2;
+				fprintf(stderr, "Error: -format arg '%s' is not a valid expression\n", argv[i+2]);
+				exit(2);
 			}
 			i+=2;
 		}
@@ -409,7 +412,8 @@ main( int argc, const char *argv[] )
 			}
 			int ixNext = parse_autoformat_args(argc, argv, i+1, pcolon, prmask, attrs, false);
 			if (ixNext < 0) {
-				return -ixNext;
+				fprintf(stderr, "Error: -autoformat arg '%s' is not a valid expression\n", argv[-ixNext]);
+				exit(2);
 			}
 			if (ixNext > i) {
 				i = ixNext-1;
