@@ -806,11 +806,11 @@ main( int argc, const char *argv[] )
 		// for these commands, double check that the current configuration files
 		// can be parsed by daemons running as 'condor' before running the command
 		// note that check_config_file_access does nothing if can_switch_ids() is false.
-		StringList errFiles;
+		std::vector<std::string> errFiles;
 		if ( ! check_config_file_access("condor", errFiles)) {
 			fprintf(stderr, "ERROR: the following configuration files cannot be read by user 'condor'. Aborting command\n");
-			for (const char * file = errFiles.first(); file != NULL; file = errFiles.next()) {
-				fprintf(stderr, "\t%s\n", file);
+			for (auto& file: errFiles) {
+				fprintf(stderr, "\t%s\n", file.c_str());
 			}
 			exit(1);
 		}
