@@ -56,6 +56,7 @@ static bool tokenize_end(void);
 static bool tokenize_empty(void);
 static bool tokenize_empty_delimiter(void);
 static bool range_base_string_token_iterator();
+static bool range_base_string_view_token_iterator();
 static bool escape_chars_length_1();
 static bool escape_chars_empty();
 static bool escape_chars_multiple();
@@ -91,6 +92,7 @@ bool FTEST_stl_string_utils(void) {
 	driver.register_function(tokenize_empty);
 	driver.register_function(tokenize_empty_delimiter);
 	driver.register_function(range_base_string_token_iterator);
+	driver.register_function(range_base_string_view_token_iterator);
 	driver.register_function(escape_chars_length_1);
 	driver.register_function(escape_chars_empty);
 	driver.register_function(escape_chars_multiple);
@@ -562,6 +564,20 @@ static bool range_base_string_token_iterator() {
 	}
 }
 
+static bool range_base_string_view_token_iterator() {
+	emit_test("Test range-based StringTokenIterator on a string_view.");
+	std::string_view in {"   abcd a b ccccc    "};
+	std::array<std::string,4>  expected {"abcd", "a", "b", "ccccc"};
+
+	bool passed = std::equal(expected.begin(), expected.end(),
+			StringTokenIterator(in).begin(),
+			StringTokenIterator(in).end());
+	if (passed) {
+		PASS;
+	} else {
+		FAIL;
+	}
+}
 static bool escape_chars_length_1() {
 	emit_test("Test EscapeChars() on a source string of length 1.");
 	std::string a = EscapeChars("z", "z", '\\');
