@@ -415,22 +415,18 @@ const char * is_attr_in_attr_list(const char * attr, const char * list)
 
 std::vector<std::string> split(const std::string& str, const char* delim, bool trim)
 {
-	int start, len;
 	std::vector<std::string> list;
-	StringTokenIterator sti(str, delim, trim);
-	while ((start = sti.next_token(len)) >= 0) {
-		list.emplace_back(&str[start], len);
+	for (const auto& item: StringTokenIterator(str, delim, trim)) {
+		list.emplace_back(item);
 	}
 	return list;
 }
 
 std::vector<std::string> split(const char* str, const char* delim, bool trim)
 {
-	int start, len;
 	std::vector<std::string> list;
-	StringTokenIterator sti(str, delim, trim);
-	while ((start = sti.next_token(len)) >= 0) {
-		list.emplace_back(&str[start], len);
+	for (const auto& item: StringTokenIterator(str, delim, trim)) {
+		list.emplace_back(item);
 	}
 	return list;
 }
@@ -546,10 +542,18 @@ matches_withwildcard_impl(const char* pattern, const char* str, bool anycase, bo
 
 		if ( (asterisk = strchr(x,'*')) == NULL ) {
 			// There is no wildcard in this entry; just compare
-			if (anycase) {
-				temp = strcasecmp(x, str);
+			if (prefix) {
+				if (anycase) {
+					temp = strncasecmp(x, str, strlen(x));
+				} else {
+					temp = strncmp(x, str, strlen(x));
+				}
 			} else {
-				temp = strcmp(x, str);
+				if (anycase) {
+					temp = strcasecmp(x, str);
+				} else {
+					temp = strcmp(x, str);
+				}
 			}
 			return (temp == MATCH);
 		}
@@ -772,12 +776,12 @@ int StringTokenIterator::next_token(int & length)
 	size_t ixEnd = ix;
 
 	// skip leading separators and whitespace (if trimming)
-	while (str[ix] && (strchr(delims, str[ix]) || (m_trim && isspace(str[ix])))) ++ix;
+	while (ix < len && str[ix] && (strchr(delims, str[ix]) || (m_trim && isspace(str[ix])))) ++ix;
 	ixNext = ix;
 	ixEnd = ix;
 
 	// scan for next delimiter or \0
-	while (str[ix] && !strchr(delims, str[ix])) {
+	while (ix < len && str[ix] && !strchr(delims, str[ix])) {
 		if (!m_trim || !isspace(str[ix])) {
 			ixEnd = ix;
 		}
