@@ -1553,15 +1553,15 @@ int store_cred_handler(int /*i*/, Stream *s)
 				// TODO: We deliberately ignore the user domains. Isn't
 				//   that a security issue?
 				// we don't allow updates to the pool password through this interface
-			StringList auth_users;
+			std::vector<std::string> auth_users;
 			param_and_insert_unique_items("CRED_SUPER_USERS", auth_users);
-			auth_users.insert(username.c_str());
+			auth_users.emplace_back(username);
 			const char *sock_owner = sock->getOwner();
 			if ( sock_owner == NULL ||
 #if defined(WIN32)
-			     !auth_users.contains_anycase_withwildcard( sock_owner )
+			     !contains_anycase_withwildcard(auth_users, sock_owner)
 #else
-			     !auth_users.contains_withwildcard( sock_owner )
+			     !contains_withwildcard(auth_users, sock_owner)
 #endif
 			   )
 			{
@@ -2585,9 +2585,8 @@ bool hasTokenSigningKey(const std::string &key_id, CondorError *err) {
 
 	// do a quick check in the issuer name cache, but don't rebuild it
 	auto keys = g_issuer_name_cache.Peek();
-	if ( ! keys.empty()) {
-		StringList list(keys.c_str());
-		if (list.contains(key_id.c_str())) {
+	for (auto& item: StringTokenIterator(keys)) {
+		if (item == key_id) {
 			return true;
 		}
 	}
