@@ -46,9 +46,11 @@
 #define SUBMIT_KEY_Prio "prio"
 #define SUBMIT_KEY_Notification "notification"
 #define SUBMIT_KEY_Executable "executable"
+#define SUBMIT_KEY_INTERACTIVE_Executable "interactive_exectuable"
 #define SUBMIT_KEY_Description "description"
 #define SUBMIT_KEY_Arguments1 "arguments"
 #define SUBMIT_KEY_Arguments2 "arguments2"
+#define SUBMIT_KEY_INTERACTIVE_Args "interactive_args"
 #define SUBMIT_KEY_Environment "environment"
 #define SUBMIT_KEY_Env "env"
 #define SUBMIT_KEY_Environment2 "environment2"
@@ -76,6 +78,11 @@
 #define SUBMIT_KEY_RequireGpus "require_gpus"
 #define SUBMIT_KEY_RequestPrefix "request_"
 #define SUBMIT_KEY_RequirePrefix "require_"
+// GPU property constraint values
+#define SUBMIT_KEY_GpusMinMemory "gpus_minimum_memory"
+#define SUBMIT_KEY_GpusMinCapability "gpus_minimum_capability"
+#define SUBMIT_KEY_GpusMaxCapability "gpus_maximum_capability"
+#define SUBMIT_KEY_GpusMinRuntime "gpus_minimum_runtime"
 
 #define SUBMIT_KEY_Universe "universe"
 #define SUBMIT_KEY_MachineCount "machine_count"
@@ -262,6 +269,7 @@
 #define SUBMIT_KEY_DockerImage "docker_image"
 #define SUBMIT_KEY_DockerNetworkType "docker_network_type"
 #define SUBMIT_KEY_DockerPullPolicy "docker_pull_policy"
+#define SUBMIT_KEY_DockerOverrideEntrypoint "docker_override_entrypoint"
 
 #define SUBMIT_KEY_ContainerImage "container_image"
 #define SUBMIT_KEY_ContainerServiceNames "container_service_names"
@@ -836,6 +844,7 @@ protected:
 	int SetTransferFiles();
 	int FixupTransferInputFiles();
 	int SetRequirements(); // after SetTransferFiles
+	int SetResourceRequirements(); // after SetRequestResources
 
 	int SetForcedSubmitAttrs(); // set +Attrib (MY.Attrib) values from SUBMIT_ATTRS directly into the job ad. this should be called second to last
 	int SetForcedAttributes();	// set +Attrib (MY.Attrib) hashtable keys directly into the job ad.  this should be called last.
