@@ -30,7 +30,6 @@
 #include "match_prefix.h"    // is_arg_colon_prefix
 #include "condor_api.h"
 #include "condor_string.h"
-#include "status_types.h"
 #include "directory.h"
 #include "format_time.h"
 #include "console-utils.h"
@@ -88,7 +87,7 @@ static struct AppType {
 	List<const char> target_names;    // list of target names to query
 	List<LOG_INFO>   all_log_info;    // pool of info from scanning log directories.
 
-	List<const char> print_head; // The list of headings for the mask entries
+	std::vector<const char *> print_head; // The list of headings for the mask entries
 	AttrListPrintMask print_mask;
 	classad::References projection;    // Attributes that we want the server to send us
 
@@ -155,7 +154,7 @@ static struct AppType {
 	~AppType() {
 		target_names.Clear();
 		all_log_info.Clear();
-		print_head.Clear();
+		print_head.clear();
 		for (char *p : query_addrs) {
 			free(p);
 		}
@@ -315,7 +314,7 @@ void AddPrintColumn(const char * heading, int width, const char * expr)
 		exit(1);
 	}
 
-	App.print_head.Append(heading);
+	App.print_head.emplace_back(heading);
 
 	int wid = (int)(width ? width : strlen(heading));
 	int opts = FormatOptionNoTruncate | FormatOptionAutoWidth;
@@ -1038,7 +1037,7 @@ format_jobid_program (const char *jobid, Formatter & /*fmt*/)
 void AddPrintColumn(const char * heading, int width, const char * attr, const CustomFormatFn & fmt)
 {
 	App.projection.insert(attr);
-	App.print_head.Append(heading);
+	App.print_head.emplace_back(heading);
 
 	int wid = width ? width : (int)strlen(heading);
 	int opts = FormatOptionNoTruncate | FormatOptionAutoWidth;
@@ -1226,11 +1225,11 @@ void parse_args(int /*argc*/, char *argv[])
 					std::string lbl = "";
 					int wid = 0;
 					int opts = FormatOptionNoTruncate;
-					if (fheadings || App.print_head.Length() > 0) {
+					if (fheadings || App.print_head.size() > 0) {
 						const char * hd = fheadings ? parg : "(expr)";
 						wid = 0 - (int)strlen(hd);
 						opts = FormatOptionAutoWidth | FormatOptionNoTruncate;
-						App.print_head.Append(hd);
+						App.print_head.emplace_back(hd);
 					}
 					else if (flabel) { formatstr(lbl,"%s = ", parg); wid = 0; opts = 0; }
 
